@@ -18,7 +18,7 @@ Not implemented yet:
     * command line args
       - placeholders are ignored
 """
-
+import atexit
 import ConfigParser
 import errno
 import grp
@@ -35,13 +35,14 @@ try:
 except ImportError:
     rpmutils_present = False
 import re
+import shutil
 import subprocess
 import tempfile
 
 # Override BOTO_CONFIG, which makes boto look only at the specified
 # config file, instead of the default locations
 os.environ['BOTO_CONFIG'] = '/var/lib/heat-cfntools/cfn-boto-cfg'
-from boto.cloudformation import CloudFormationConnection
+from boto import cloudformation
 
 
 LOG = logging.getLogger(__name__)
@@ -53,8 +54,8 @@ def to_boolean(b):
 
 
 def parse_creds_file(path='/etc/cfn/cfn-credentials'):
-    '''
-    Parse the cfn credentials file
+    '''Parse the cfn credentials file.
+
     Default location is as specified, and it is expected to contain
     exactly two keys "AWSAccessKeyId" and "AWSSecretKey)
     The two keys are returned a dict (if found)
@@ -116,7 +117,7 @@ class HupConfig(object):
         resources = []
         for h in self.hooks:
             r = h.resource_name_get()
-            if not r in resources:
+            if r not in resources:
                 resources.append(h.resource_name_get())
         return resources
 
@@ -151,9 +152,7 @@ class Hook(object):
 
 
 class CommandRunner(object):
-    """
-    Helper class to run a command and store the output.
-    """
+    """Helper class to run a command and store the output."""
 
     def __init__(self, command, nextcommand=None):
         self._command = command
@@ -174,8 +173,7 @@ class CommandRunner(object):
         return s
 
     def run(self, user='root', cwd=None, env=None):
-        """
-        Run the Command and return the output.
+        """Run the Command and return the output.
 
         Returns:
             self
@@ -216,15 +214,12 @@ class RpmHelper(object):
 
     @classmethod
     def prepcache(cls):
-        """
-        Prepare the yum cache
-        """
+        """Prepare the yum cache."""
         CommandRunner("yum -y makecache").run()
 
     @classmethod
     def compare_rpm_versions(cls, v1, v2):
-        """
-        Compare two RPM version strings.
+        """Compare two RPM version strings.
 
         Arguments:
             v1 -- a version string
@@ -246,8 +241,7 @@ class RpmHelper(object):
 
     @classmethod
     def newest_rpm_version(cls, versions):
-        """
-        Returns the highest (newest) version from a list of versions.
+        """Returns the highest (newest) version from a list of versions.
 
         Arguments:
             versions -- A list of version strings
@@ -264,8 +258,7 @@ class RpmHelper(object):
 
     @classmethod
     def rpm_package_version(cls, pkg):
-        """
-        Returns the version of an installed RPM.
+        """Returns the version of an installed RPM.
 
         Arguments:
             pkg -- A package name
@@ -276,8 +269,7 @@ class RpmHelper(object):
 
     @classmethod
     def rpm_package_installed(cls, pkg):
-        """
-        Indicates whether pkg is in rpm database.
+        """Indicates whether pkg is in rpm database.
 
         Arguments:
             pkg -- A package name (with optional version and release spec).
@@ -290,8 +282,7 @@ class RpmHelper(object):
 
     @classmethod
     def yum_package_available(cls, pkg):
-        """
-        Indicates whether pkg is available via yum
+        """Indicates whether pkg is available via yum.
 
         Arguments:
             pkg -- A package name (with optional version and release spec).
@@ -305,8 +296,7 @@ class RpmHelper(object):
 
     @classmethod
     def install(cls, packages, rpms=True):
-        """
-        Installs (or upgrades) a set of packages via RPM or via Yum.
+        """Installs (or upgrades) a set of packages via RPM or via Yum.
 
         Arguments:
             packages -- a list of packages to install
@@ -335,8 +325,7 @@ class RpmHelper(object):
 
     @classmethod
     def downgrade(cls, packages, rpms=True):
-        """
-        Downgrades a set of packages via RPM or via Yum.
+        """Downgrades a set of packages via RPM or via Yum.
 
         Arguments:
             packages -- a list of packages to downgrade
@@ -384,9 +373,7 @@ class PackagesHandler(object):
         self._packages = packages
 
     def _handle_gem_packages(self, packages):
-        """
-        very basic support for gems
-        """
+        """very basic support for gems."""
         # TODO(asalkeld) support versions
         # -b == local & remote install
         # -y == install deps
@@ -401,18 +388,14 @@ class PackagesHandler(object):
                 CommandRunner('gem install %s %s' % (opts, pkg_name)).run()
 
     def _handle_python_packages(self, packages):
-        """
-        very basic support for easy_install
-        """
+        """very basic support for easy_install."""
         # TODO(asalkeld) support versions
         for pkg_name, versions in packages.iteritems():
             cmd_str = 'easy_install %s' % (pkg_name)
             CommandRunner(cmd_str).run()
 
     def _handle_yum_packages(self, packages):
-        """
-        Handle installation, upgrade, or downgrade of a set of
-        packages via yum.
+        """Handle installation, upgrade, or downgrade of packages via yum.
 
         Arguments:
         packages -- a package entries map of the form:
@@ -459,9 +442,7 @@ class PackagesHandler(object):
                 RpmHelper.downgrade(downgrades)
 
     def _handle_rpm_packages(self, packages):
-        """
-        Handle installation, upgrade, or downgrade of a set of
-        packages via rpm.
+        """Handle installation, upgrade, or downgrade of packages via rpm.
 
         Arguments:
         packages -- a package entries map of the form:
@@ -476,9 +457,7 @@ class PackagesHandler(object):
         pass
 
     def _handle_apt_packages(self, packages):
-        """
-        very basic support for apt
-        """
+        """very basic support for apt."""
         # TODO(asalkeld) support versions
         pkg_list = ' '.join([p for p in packages])
 
@@ -499,8 +478,8 @@ class PackagesHandler(object):
         return handler
 
     def apply_packages(self):
-        """
-        Install, upgrade, or downgrade packages listed
+        """Install, upgrade, or downgrade packages listed.
+
         Each package is a dict containing package name and a list of versions
         Install order:
           * dpkg
@@ -558,14 +537,14 @@ class FilesHandler(object):
                 try:
                     user_info = pwd.getpwnam(meta['owner'])
                     uid = user_info[2]
-                except KeyError as ex:
+                except KeyError:
                     pass
 
             if 'group' in meta:
                 try:
                     group_info = grp.getgrnam(meta['group'])
                     gid = group_info[2]
-                except KeyError as ex:
+                except KeyError:
                     pass
 
             os.chown(dest, uid, gid)
@@ -574,9 +553,7 @@ class FilesHandler(object):
 
 
 class SourcesHandler(object):
-    '''
-    tar, tar+gzip,tar+bz2 and zip
-    '''
+    '''tar, tar+gzip,tar+bz2 and zip.'''
     _sources = {}
 
     def __init__(self, sources):
@@ -584,6 +561,7 @@ class SourcesHandler(object):
 
     def _url_to_tmp_filename(self, url):
         tempdir = tempfile.mkdtemp()
+        atexit.register(lambda: shutil.rmtree(tempdir, True))
         sp = url.split('/')
         if 'https://github.com' in url:
             if 'zipball' == sp[-2]:
@@ -595,49 +573,60 @@ class SourcesHandler(object):
 
         return '%s/%s' % (tempdir, sp[-1])
 
-    def _decompress(self, archive, dest_dir):
-        cmd_str = ''
-        LOG.debug("Decompressing")
-        (r, ext) = os.path.splitext(archive)
-        if ext == '.tgz':
-            cmd_str = 'tar -C %s -xzf %s' % (dest_dir, archive)
-        elif ext == '.tbz2':
-            cmd_str = 'tar -C %s -xjf %s' % (dest_dir, archive)
-        elif ext == '.zip':
-            cmd_str = 'unzip -d %s %s' % (dest_dir, archive)
-        elif ext == '.tar':
-            cmd_str = 'tar -C %s -xf %s' % (dest_dir, archive)
-        elif ext == '.gz':
-            (r, ext) = os.path.splitext(r)
-            if ext:
-                cmd_str = 'tar -C %s -xzf %s' % (dest_dir, archive)
-            else:
-                cmd_str = 'gunzip -c %s > %s' % (archive, dest_dir)
-        elif ext == 'bz2':
-            (r, ext) = os.path.splitext(r)
-            if ext:
-                cmd_str = 'tar -C %s -xjf %s' % (dest_dir, archive)
-            else:
-                cmd_str = 'bunzip2 -c %s > %s' % (archive, dest_dir)
+    def _splitext(self, path):
+        (r, ext) = os.path.splitext(path)
+        return (r, ext.lower())
+
+    def _source_type(self, name):
+        (r, ext) = self._splitext(name)
+        if ext == '.gz':
+            (r, ext2) = self._splitext(r)
+            if ext2 == '.tar':
+                ext = '.tgz'
+        elif ext == '.bz2':
+            (r, ext2) = self._splitext(r)
+            if ext2 == '.tar':
+                ext = '.tbz2'
+        return ext
+
+    def _apply_source_cmd(self, dest, url):
+        cmd = ""
+        basename = os.path.basename(url)
+        stype = self._source_type(basename)
+        if stype == '.tgz':
+            cmd = "wget -q -O - '%s' | gunzip | tar -xvf -" % url
+        elif stype == '.tbz2':
+            cmd = "wget -q -O - '%s' | bunzip2 | tar -xvf -" % url
+        elif stype == '.zip':
+            tmp = self._url_to_tmp_filename(url)
+            cmd = "wget -q -O '%s' '%s' && unzip -o '%s'" % (tmp, url, tmp)
+        elif stype == '.tar':
+            cmd = "wget -q -O - '%s' | tar -xvf -" % url
+        elif stype == '.gz':
+            (r, ext) = self._splitext(basename)
+            cmd = "wget -q -O - '%s' | gunzip > '%s'" % (url, r)
+        elif stype == '.bz2':
+            (r, ext) = self._splitext(basename)
+            cmd = "wget -q -O - '%s' | bunzip2 > '%s'" % (url, r)
         else:
             pass
-        return CommandRunner(cmd_str)
+
+        if cmd != '':
+            cmd = "mkdir -p '%s'; cd '%s'; %s" % (dest, dest, cmd)
+
+        return cmd
+
+    def _apply_source(self, dest, url):
+        cmd = self._apply_source_cmd(dest, url)
+        if cmd != '':
+            runner = CommandRunner(cmd)
+            runner.run()
 
     def apply_sources(self):
         if not self._sources:
             return
         for dest, url in self._sources.iteritems():
-            tmp_name = self._url_to_tmp_filename(url)
-            cmd_str = 'wget -O %s %s' % (tmp_name, url)
-            try:
-                os.makedirs(dest)
-            except OSError as e:
-                if e.errno == errno.EEXIST:
-                    LOG.debug(str(e))
-                else:
-                    LOG.exception(e)
-            decompress_command = self._decompress(tmp_name, dest)
-            CommandRunner(cmd_str, decompress_command).run()
+            self._apply_source(dest, url)
 
 
 class ServicesHandler(object):
@@ -741,9 +730,7 @@ class ServicesHandler(object):
         return handler
 
     def apply_services(self):
-        """
-        Starts, stops, enables, disables services
-        """
+        """Starts, stops, enables, disables services."""
         if not self._services:
             return
         for manager, service_entries in self._services.iteritems():
@@ -754,9 +741,7 @@ class ServicesHandler(object):
                 self._initialize_services(handler, service_entries)
 
     def monitor_services(self):
-        """
-        Restarts failed services, and runs hooks.
-        """
+        """Restarts failed services, and runs hooks."""
         if not self._services:
             return
         for manager, service_entries in self._services.iteritems():
@@ -789,9 +774,7 @@ class ConfigsetsHandler(object):
                 executionlist.append(elem)
 
     def get_configsets(self):
-        """
-        Returns a list of Configsets to execute in template
-        """
+        """Returns a list of Configsets to execute in template."""
         if not self.configsets:
             if self.selectedsets:
                 raise Exception('Template has no configSets')
@@ -818,9 +801,9 @@ class ConfigsetsHandler(object):
 
 def metadata_server_port(
         datafile='/var/lib/heat-cfntools/cfn-metadata-server'):
-    """
-    Return the the metadata server port
-    reads the :NNNN from the end of the URL in cfn-metadata-server
+    """Return the the metadata server port.
+
+    Reads the :NNNN from the end of the URL in cfn-metadata-server
     """
     try:
         f = open(datafile)
@@ -847,9 +830,7 @@ class CommandsHandler(object):
         self.commands = commands
 
     def apply_commands(self):
-        """
-        Execute commands on the instance in alphabetical order by name.
-        """
+        """Execute commands on the instance in alphabetical order by name."""
         if not self.commands:
             return
         for command_label in sorted(self.commands):
@@ -914,9 +895,7 @@ class GroupsHandler(object):
         self.groups = groups
 
     def apply_groups(self):
-        """
-        Create Linux/UNIX groups and assign group IDs
-        """
+        """Create Linux/UNIX groups and assign group IDs."""
         if not self.groups:
             return
         for group, properties in self.groups.iteritems():
@@ -960,9 +939,7 @@ class UsersHandler(object):
         self.users = users
 
     def apply_users(self):
-        """
-        Create Linux/UNIX users and assign user IDs, groups and homedir
-        """
+        """Create Linux/UNIX users and assign user IDs, groups and homedir."""
         if not self.users:
             return
         for user, properties in self.users.iteritems():
@@ -1043,9 +1020,7 @@ class Metadata(object):
         self._has_changed = False
 
     def remote_metadata(self):
-        """
-        Connect to the metadata server and retreive the metadata from there.
-        """
+        """Connect to the metadata server and retreive the metadata."""
 
         if self.credentials_file:
             credentials = parse_creds_file(self.credentials_file)
@@ -1059,10 +1034,11 @@ class Metadata(object):
 
         port = metadata_server_port() or self.DEFAULT_PORT
 
-        client = CloudFormationConnection(aws_access_key_id=access_key,
-                                          aws_secret_access_key=secret_key,
-                                          is_secure=False, port=port,
-                                          path="/v1", debug=0)
+        client = cloudformation.CloudFormationConnection(
+            aws_access_key_id=access_key,
+            aws_secret_access_key=secret_key,
+            is_secure=False, port=port,
+            path="/v1", debug=0)
 
         res = client.describe_stack_resource(self.stack, self.resource)
         # Note pending upstream patch will make this response a
@@ -1078,8 +1054,11 @@ class Metadata(object):
             meta_str=None,
             default_path='/var/lib/heat-cfntools/cfn-init-data',
             last_path='/var/cache/heat-cfntools/last_metadata'):
-        """
-        Read the metadata from the given filename
+        """Read the metadata from the given filename or from the remote server.
+
+           Returns:
+               True -- success
+              False -- error
         """
         if meta_str:
             self._data = meta_str
@@ -1141,24 +1120,32 @@ class Metadata(object):
         cache_dir = os.path.dirname(last_path)
         if not os.path.isdir(cache_dir):
             try:
-                os.makedirs(cache_dir, mode=0700)
+                os.makedirs(cache_dir, mode=0o700)
             except IOError as e:
                 LOG.warn('could not create metadata cache dir %s [%s]' %
                          (cache_dir, e))
                 return
         # save current metadata to file
-        tmp_mdpath = last_path
-        with open(tmp_mdpath, 'w+') as cf:
-            os.chmod(tmp_mdpath, 0600)
+        tmp_dir = os.path.dirname(last_path)
+        with tempfile.NamedTemporaryFile(dir=tmp_dir,
+                                         mode='wb',
+                                         delete=False) as cf:
+            os.chmod(cf.name, 0o600)
             cf.write(json.dumps(self._metadata))
+            os.rename(cf.name, last_path)
+
+        return True
 
     def __str__(self):
         return json.dumps(self._metadata)
 
+    def display(self):
+        if self._metadata is not None:
+            print str(self)
+        return
+
     def _is_valid_metadata(self):
-        """
-        Should find the AWS::CloudFormation::Init json key
-        """
+        """Should find the AWS::CloudFormation::Init json key."""
         is_valid = self._metadata and \
             self._init_key in self._metadata and \
             self._metadata[self._init_key]
@@ -1167,8 +1154,8 @@ class Metadata(object):
         return is_valid
 
     def _process_config(self, config="config"):
-        """
-        Parse and process a config section
+        """Parse and process a config section.
+
           * packages
           * sources
           * groups
@@ -1192,9 +1179,7 @@ class Metadata(object):
         ServicesHandler(self._config.get("services")).apply_services()
 
     def cfn_init(self):
-        """
-        Process the resource metadata
-        """
+        """Process the resource metadata."""
         if not self._is_valid_metadata():
             raise Exception("invalid metadata")
         else:
@@ -1207,18 +1192,17 @@ class Metadata(object):
                     self._process_config(item)
 
     def cfn_hup(self, hooks):
-        """
-        Process the resource metadata
-        """
+        """Process the resource metadata."""
         if not self._is_valid_metadata():
-            LOG.info('Metadata does not contain a %s section' % self._init_key)
-        else:
-            if self._is_local_metadata:
-                self._config = self._metadata["config"]
-                s = self._config.get("services")
-                sh = ServicesHandler(s, resource=self.resource, hooks=hooks)
-                sh.monitor_services()
+            LOG.debug(
+                'Metadata does not contain a %s section' % self._init_key)
 
-            if self._has_changed:
-                for h in hooks:
-                    h.event('post.update', self.resource, self.resource)
+        if self._is_local_metadata:
+            self._config = self._metadata.get("config", {})
+            s = self._config.get("services")
+            sh = ServicesHandler(s, resource=self.resource, hooks=hooks)
+            sh.monitor_services()
+
+        if self._has_changed:
+            for h in hooks:
+                h.event('post.update', self.resource, self.resource)
