@@ -15,6 +15,7 @@
 # under the License.
 
 import boto.cloudformation as cfn
+import fixtures
 import json
 import mox
 import os
@@ -338,24 +339,21 @@ region=region1
 credential-file=%s-invalid
 interval=120''' % fcreds.name)
         main_conf.flush()
-        self.assertRaisesRegexp(
-            Exception,
-            'invalid credentials file',
-            cfn_helper.HupConfig,
-            [open(main_conf.name)])
-
+        e = self.assertRaises(Exception, cfn_helper.HupConfig,
+                              [open(main_conf.name)])
+        self.assertIn('invalid credentials file', str(e))
         fcreds.close()
 
     def test_hup_config(self):
-        self.mock_cmd_run(['su', 'root', '-c', '/bin/hook2']).AndReturn(
-            FakePOpen('All good'))
-        self.mock_cmd_run(['su', 'root', '-c', '/bin/hook1']).AndReturn(
-            FakePOpen('All good'))
-        self.mock_cmd_run(['su', 'root', '-c', '/bin/hook3']).AndReturn(
-            FakePOpen('All good'))
         self.mock_cmd_run(
             ['su', 'root', '-c', '/bin/cfn-http-restarted']).AndReturn(
                 FakePOpen('All good'))
+        self.mock_cmd_run(['su', 'root', '-c', '/bin/hook1']).AndReturn(
+            FakePOpen('All good'))
+        self.mock_cmd_run(['su', 'root', '-c', '/bin/hook2']).AndReturn(
+            FakePOpen('All good'))
+        self.mock_cmd_run(['su', 'root', '-c', '/bin/hook3']).AndReturn(
+            FakePOpen('All good'))
         self.m.ReplayAll()
 
         hooks_conf = tempfile.NamedTemporaryFile()
@@ -407,29 +405,31 @@ interval=120''' % fcreds.name)
             open(main_conf.name),
             open(hooks_conf.name)])
         unique_resources = mainconfig.unique_resources_get()
-        self.assertSequenceEqual([
-            'resource2',
+        self.assertThat([
+            'resource',
             'resource1',
+            'resource2',
             'resource3',
-            'resource'
-        ], unique_resources)
+        ], ttm.Equals(sorted(unique_resources)))
 
-        hooks = mainconfig.hooks
+        hooks = sorted(mainconfig.hooks,
+                       key=lambda hook: hook.resource_name_get())
+        self.assertEqual(len(hooks), 4)
         self.assertEqual(
-            '{hook2, service2.restarted, Resources.resource2.Metadata,'
-            ' root, /bin/hook2}', str(hooks[0]))
+            '{cfn-http-restarted, service.restarted,'
+            ' Resources.resource.Metadata, root, /bin/cfn-http-restarted}',
+            str(hooks[0]))
         self.assertEqual(
             '{hook1, service1.restarted, Resources.resource1.Metadata,'
             ' root, /bin/hook1}', str(hooks[1]))
         self.assertEqual(
-            '{hook3, service3.restarted, Resources.resource3.Metadata,'
-            ' root, /bin/hook3}', str(hooks[2]))
+            '{hook2, service2.restarted, Resources.resource2.Metadata,'
+            ' root, /bin/hook2}', str(hooks[2]))
         self.assertEqual(
-            '{cfn-http-restarted, service.restarted,'
-            ' Resources.resource.Metadata, root, /bin/cfn-http-restarted}',
-            str(hooks[3]))
+            '{hook3, service3.restarted, Resources.resource3.Metadata,'
+            ' root, /bin/hook3}', str(hooks[3]))
 
-        for hook in mainconfig.hooks:
+        for hook in hooks:
             hook.event(hook.triggers, None, hook.resource_name_get())
 
         hooks_conf.close()
@@ -493,7 +493,7 @@ class TestCfnHelper(testtools.TestCase):
                 fcreds.write(file_contents)
                 fcreds.flush()
                 creds = cfn_helper.parse_creds_file(fcreds.name)
-                self.assertDictEqual(creds_match, creds)
+                self.assertThat(creds_match, ttm.Equals(creds))
         parse_creds_test(
             'AWSAccessKeyId=foo\nAWSSecretKey=bar\n',
             {'AWSAccessKeyId': 'foo', 'AWSSecretKey': 'bar'}
@@ -510,6 +510,11 @@ class TestCfnHelper(testtools.TestCase):
 
 class TestMetadataRetrieve(testtools.TestCase):
 
+    def setUp(self):
+        super(TestMetadataRetrieve, self).setUp()
+        self.tdir = self.useFixture(fixtures.TempDir())
+        self.last_file = os.path.join(self.tdir.path, 'last_metadata')
+
     def test_metadata_retrieve_files(self):
 
         md_data = {"AWS::CloudFormation::Init": {"config": {"files": {
@@ -518,58 +523,66 @@ class TestMetadataRetrieve(testtools.TestCase):
 
         md = cfn_helper.Metadata('teststack', None)
 
-        with tempfile.NamedTemporaryFile() as last_file:
-            pass
-
         with tempfile.NamedTemporaryFile(mode='w+') as default_file:
             default_file.write(md_str)
             default_file.flush()
             self.assertThat(default_file.name, ttm.FileContains(md_str))
 
-            md.retrieve(
-                default_path=default_file.name,
-                last_path=last_file.name)
+            self.assertTrue(
+                md.retrieve(default_path=default_file.name,
+                            last_path=self.last_file))
 
-            self.assertThat(last_file.name, ttm.FileContains(md_str))
-            self.assertDictEqual(md_data, md._metadata)
+            self.assertThat(self.last_file, ttm.FileContains(md_str))
+            self.assertThat(md_data, ttm.Equals(md._metadata))
 
         md = cfn_helper.Metadata('teststack', None)
-        md.retrieve(
-            default_path=default_file.name,
-            last_path=last_file.name)
-        self.assertDictEqual(md_data, md._metadata)
+        self.assertTrue(md.retrieve(default_path=default_file.name,
+                                    last_path=self.last_file))
+        self.assertThat(md_data, ttm.Equals(md._metadata))
 
     def test_metadata_retrieve_none(self):
 
         md = cfn_helper.Metadata('teststack', None)
-        with tempfile.NamedTemporaryFile() as last_file:
-            pass
-        with tempfile.NamedTemporaryFile() as default_file:
-            pass
+        default_file = os.path.join(self.tdir.path, 'default_file')
 
-        md.retrieve(
-            default_path=default_file.name,
-            last_path=last_file.name)
+        self.assertFalse(md.retrieve(default_path=default_file,
+                                     last_path=self.last_file))
         self.assertIsNone(md._metadata)
 
+        displayed = self.useFixture(fixtures.StringStream('stdout'))
+        fake_stdout = displayed.stream
+        self.useFixture(fixtures.MonkeyPatch('sys.stdout', fake_stdout))
+        md.display()
+        fake_stdout.flush()
+        self.assertEqual(displayed.getDetails()['stdout'].as_text(), "")
+
     def test_metadata_retrieve_passed(self):
 
         md_data = {"AWS::CloudFormation::Init": {"config": {"files": {
             "/tmp/foo": {"content": "bar"}}}}}
         md_str = json.dumps(md_data)
 
-        with tempfile.NamedTemporaryFile() as last_file:
-            pass
+        md = cfn_helper.Metadata('teststack', None)
+        self.assertTrue(md.retrieve(meta_str=md_str,
+                                    last_path=self.last_file))
+        self.assertThat(md_data, ttm.Equals(md._metadata))
 
         md = cfn_helper.Metadata('teststack', None)
-        md.retrieve(meta_str=md_str, last_path=last_file.name)
-        self.assertDictEqual(md_data, md._metadata)
-
-        md = cfn_helper.Metadata('teststack', None)
-        md.retrieve(meta_str=md_data, last_path=last_file.name)
-        self.assertDictEqual(md_data, md._metadata)
+        self.assertTrue(md.retrieve(meta_str=md_data,
+                                    last_path=self.last_file))
+        self.assertThat(md_data, ttm.Equals(md._metadata))
         self.assertEqual(md_str, str(md))
 
+        displayed = self.useFixture(fixtures.StringStream('stdout'))
+        fake_stdout = displayed.stream
+        self.useFixture(fixtures.MonkeyPatch('sys.stdout', fake_stdout))
+        md.display()
+        fake_stdout.flush()
+        self.assertEqual(displayed.getDetails()['stdout'].as_text(),
+                         "{\"AWS::CloudFormation::Init\": {\"config\": {"
+                         "\"files\": {\"/tmp/foo\": {\"content\": \"bar\"}"
+                         "}}}}\n")
+
     def test_metadata_creates_cache(self):
         temp_home = tempfile.mkdtemp()
 
@@ -588,27 +601,26 @@ class TestMetadataRetrieve(testtools.TestCase):
 
         self.assertFalse(os.path.exists(last_path),
                          "last_metadata file already exists")
-        md.retrieve(meta_str=md_str, last_path=last_path)
+        self.assertTrue(md.retrieve(meta_str=md_str, last_path=last_path))
         self.assertTrue(os.path.exists(last_path),
                         "last_metadata file should exist")
         # Ensure created dirs and file have right perms
-        self.assertTrue(os.stat(last_path).st_mode & 0600 == 0600)
+        self.assertTrue(os.stat(last_path).st_mode & 0o600 == 0o600)
         self.assertTrue(
-            os.stat(os.path.dirname(last_path)).st_mode & 0700 == 0700)
+            os.stat(os.path.dirname(last_path)).st_mode & 0o700 == 0o700)
 
     def test_is_valid_metadata(self):
         md_data = {"AWS::CloudFormation::Init": {"config": {"files": {
             "/tmp/foo": {"content": "bar"}}}}}
 
-        with tempfile.NamedTemporaryFile() as last_file:
-            pass
         md = cfn_helper.Metadata('teststack', None)
-        md.retrieve(meta_str=md_data, last_path=last_file.name)
+        self.assertTrue(
+            md.retrieve(meta_str=md_data, last_path=self.last_file))
 
-        self.assertDictEqual(md_data, md._metadata)
+        self.assertThat(md_data, ttm.Equals(md._metadata))
         self.assertTrue(md._is_valid_metadata())
-        self.assertDictEqual(
-            md_data['AWS::CloudFormation::Init'], md._metadata)
+        self.assertThat(
+            md_data['AWS::CloudFormation::Init'], ttm.Equals(md._metadata))
 
     def test_remote_metadata(self):
 
@@ -627,25 +639,22 @@ class TestMetadataRetrieve(testtools.TestCase):
 
         m.ReplayAll()
 
-        with tempfile.NamedTemporaryFile() as last_file:
-            pass
-
         try:
             md = cfn_helper.Metadata(
                 'teststack',
                 None,
                 access_key='foo',
                 secret_key='bar')
-            md.retrieve(last_path=last_file.name)
-            self.assertDictEqual(md_data, md._metadata)
+            self.assertTrue(md.retrieve(last_path=self.last_file))
+            self.assertThat(md_data, ttm.Equals(md._metadata))
 
             with tempfile.NamedTemporaryFile(mode='w') as fcreds:
                 fcreds.write('AWSAccessKeyId=foo\nAWSSecretKey=bar\n')
                 fcreds.flush()
                 md = cfn_helper.Metadata(
                     'teststack', None, credentials_file=fcreds.name)
-                md.retrieve(last_path=last_file.name)
-            self.assertDictEqual(md_data, md._metadata)
+                self.assertTrue(md.retrieve(last_path=self.last_file))
+            self.assertThat(md_data, ttm.Equals(md._metadata))
 
             m.VerifyAll()
         finally:
@@ -653,15 +662,13 @@ class TestMetadataRetrieve(testtools.TestCase):
 
     def test_cfn_init(self):
 
-        with tempfile.NamedTemporaryFile() as last_file:
-            pass
-
         with tempfile.NamedTemporaryFile(mode='w+') as foo_file:
             md_data = {"AWS::CloudFormation::Init": {"config": {"files": {
                 foo_file.name: {"content": "bar"}}}}}
 
             md = cfn_helper.Metadata('teststack', None)
-            md.retrieve(meta_str=md_data, last_path=last_file.name)
+            self.assertTrue(
+                md.retrieve(meta_str=md_data, last_path=self.last_file))
             md.cfn_init()
             self.assertThat(foo_file.name, ttm.FileContains('bar'))
 
@@ -678,10 +685,10 @@ class TestSourcesHandler(MockPopenTestCase):
         td = os.path.dirname(end_file)
         self.m.StubOutWithMock(tempfile, 'mkdtemp')
         tempfile.mkdtemp().AndReturn(td)
-        cmd = ['su', 'root', '-c', 'wget -O %s %s' % (end_file, url)]
+        er = "mkdir -p '%s'; cd '%s'; wget -q -O - '%s' | gunzip | tar -xvf -"
+        cmd = ['su', 'root', '-c',
+               er % (dest, dest, url)]
         self.mock_cmd_run(cmd).AndReturn(FakePOpen('Wget good'))
-        cmd = ['su', 'root', '-c', 'tar -C %s -xzf %s' % (dest, end_file)]
-        self.mock_cmd_run(cmd).AndReturn(FakePOpen('Tarball good'))
         self.m.ReplayAll()
         sh = cfn_helper.SourcesHandler(sources)
         sh.apply_sources()
@@ -699,3 +706,47 @@ class TestSourcesHandler(MockPopenTestCase):
         self.addCleanup(os.rmdir, td)
         end_file = '%s/file.tar.gz' % td
         self._test_apply_sources(url, end_file)
+
+    def test_apply_source_cmd(self):
+        sh = cfn_helper.SourcesHandler({})
+        er = "mkdir -p '%s'; cd '%s'; wget -q -O - '%s' | %s | tar -xvf -"
+        dest = '/tmp'
+        # test tgz
+        url = 'http://www.example.com/a.tgz'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, url, "gunzip"), cmd)
+        # test tar.gz
+        url = 'http://www.example.com/a.tar.gz'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, url, "gunzip"), cmd)
+        # test tbz2
+        url = 'http://www.example.com/a.tbz2'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, url, "bunzip2"), cmd)
+        # test tar.bz2
+        url = 'http://www.example.com/a.tar.bz2'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, url, "bunzip2"), cmd)
+        # test zip
+        er = "mkdir -p '%s'; cd '%s'; wget -q -O '%s' '%s' && unzip -o '%s'"
+        url = 'http://www.example.com/a.zip'
+        d = "/tmp/tmp2I0yNK"
+        tmp = "%s/a.zip" % d
+        self.m.StubOutWithMock(tempfile, 'mkdtemp')
+        tempfile.mkdtemp().AndReturn(d)
+        self.m.ReplayAll()
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, tmp, url, tmp), cmd)
+        # test gz
+        er = "mkdir -p '%s'; cd '%s'; wget -q -O - '%s' | %s > '%s'"
+        url = 'http://www.example.com/a.sh.gz'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, url, "gunzip", "a.sh"), cmd)
+        # test bz2
+        url = 'http://www.example.com/a.sh.bz2'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual(er % (dest, dest, url, "bunzip2", "a.sh"), cmd)
+        # test other
+        url = 'http://www.example.com/a.sh'
+        cmd = sh._apply_source_cmd(dest, url)
+        self.assertEqual("", cmd)
